"""
Actions for filtering based on temporal information
"""

from calendar import monthrange
from datetime import date, datetime
from logging import getLogger
from typing import Iterable

from spotify_model import ReleaseDatePrecision, SearchAlbum


def temporal_convert(date_str: str, precision: ReleaseDatePrecision) -> date:
    """
    For date strings that don't have date-level precision, the date is treated as the final day within that period;
    thus, albums released in "1981" are effectively released on 1981-12-31, and albums released in "1981-07" are
    treated as "1981-07-31"
    """

    if precision == ReleaseDatePrecision.YEAR:
        actual = datetime.strptime(date_str, "%Y")
        effective = date(actual.year, 12, 31)
    elif precision == ReleaseDatePrecision.MONTH:
        actual = datetime.strptime(date_str, "%Y-%m")
        final_day = monthrange(actual.year, actual.month)[1] - 1
        effective = date(actual.year, actual.month, final_day)
    else:
        effective = datetime.strptime(date_str, "%Y-%m-%d").date()

    return effective


def temporal_released_after(albums: Iterable[SearchAlbum], released_after: date) -> Iterable[SearchAlbum]:
    """
    Filter albums to after a specific release date.

    For albums that don't have date-level precision, the release date is treated as the final day within that period;
    thus, albums released in "1981" are effectively released on 1981-12-31, and albums released in "1981-07" are
    treated as "1981-07-31"
    """

    logger = getLogger(__name__)

    for album in albums:
        effective_release = temporal_convert(album.release_date, album.release_date_precision)

        if effective_release >= released_after:
            logger.debug(
                "Including album=%s released on date=%s (prior to date=%s)",
                album.name,
                effective_release,
                released_after,
            )
            yield album
        else:
            logger.debug(
                "Skipping album=%s released on date=%s (prior to date=%s)",
                album.name,
                effective_release,
                released_after,
            )
