"""
Utility methods for working with the Spotify API
"""
from math import ceil
from pathlib import Path
from typing import Any, Dict, Iterable, List, Optional, Protocol, TypeVar

import yaml
from spotify_model import Paging
from spotipy import Spotify, SpotifyClientCredentials, SpotifyOAuth

DEFAULT_LIMIT = 50

T = TypeVar("T")  # pylint: disable=invalid-name


def read_credentials_server(path: Path) -> Spotify:
    "Read credentials from a YAML file and construct a Spotify client using the server workflow"

    with open(path, "r") as credentials_file:
        credentials = yaml.safe_load(credentials_file)
        credentials_manager = SpotifyClientCredentials(credentials["client_id"], credentials["client_secret"])
        return Spotify(client_credentials_manager=credentials_manager)


def read_credentials_oauth(path: Path, redirect_uri: str, scopes: List[str]) -> Spotify:
    "Read credentials from a YAML file and authorize a user"

    with open(path, "r") as credentials_file:
        credentials = yaml.safe_load(credentials_file)
        credentials_manager = SpotifyOAuth(
            credentials["client_id"], credentials["client_secret"], redirect_uri, scope=scopes
        )
        return Spotify(client_credentials_manager=credentials_manager)


class Paginated(Protocol):
    "Protocol definition for functions that will be provided to the `exhaust` handler"

    def __call__(self, limit: int, offset: int) -> Paging:
        ...


def exhaust(
    function: Paginated, initial: Optional[Paging] = None, limit: int = DEFAULT_LIMIT
) -> Iterable[Dict[str, Any]]:
    "Exhaust a function that returns a pagination object"
    response = initial if initial is not None else function(limit=limit, offset=0)

    for item in response.items:
        yield item

    for i in range(1, ceil(response.total / limit)):
        response = function(limit=response.limit, offset=response.limit * i)
        for item in response.items:
            yield item


def chunk(items: Iterable[T], size: int) -> Iterable[List[T]]:
    "Split an iterable into smaller chunks"

    assert size >= 1

    return_items = []
    for item in items:
        return_items.append(item)

        if len(return_items) == size:
            yield return_items
            return_items = []

    if return_items:
        yield return_items
